require "oauth2"

module GoSso
  autoload :User, 'go_sso/user'
  autoload :Current, 'go_sso/current'

  module Controllers
    autoload :Helpers, 'go_sso/controllers/helpers'
  end

  mattr_accessor :client_id
  mattr_accessor :client_secret
  mattr_accessor :site
  mattr_accessor :user_json_url, default: 'api/me'
  mattr_accessor :user_cache_ttl, default: 1.minute
  mattr_accessor :main_app_module_name
  mattr_accessor :host
  mattr_accessor :fake_user_json
  mattr_accessor :after_login, default: ->(ctx){}
  mattr_accessor :routes_prefix, default: "/go_sso"

  def self.setup
    yield self
  end

  def self.set_default_options!
    self.client_id = ENV['GO_SSO_CLIENT_ID']
    self.client_secret = ENV['GO_SSO_CLIENT_SECRET']
    self.site = ENV['GO_SSO_CLIENT_SITE']
    self.main_app_module_name = if Rails::VERSION::MAJOR == 6
      Rails.application.class.module_parent.to_s
    else
      Rails.application.class.parent.to_s
    end
  end

  def self.client
    @client ||= OAuth2::Client.new(client_id, client_secret, site: site)
  end

  def self.authorize_url
    client.auth_code.authorize_url(redirect_uri: Rails.application.routes.url_helpers.go_sso_callback_url(host: current_host))
  end

  def self.get_token(code)
    client.auth_code.get_token(code, redirect_uri: Rails.application.routes.url_helpers.go_sso_callback_url(host: current_host))
  end

  def self.current_host
    host || Current.host
  end

  def self.get_user_json(token = nil)
    return fake_user_json.as_json if test_mode?
    o_token = OAuth2::AccessToken.new(client, token)
    json_str = Rails.cache.fetch([:go_sso_user, token], expires_in: user_cache_ttl) do
      o_token.get(GoSso.user_json_url).body
    end
    JSON.parse(json_str)
  rescue Faraday::ConnectionFailed
    raise GoSso::FailedToOpenConnection
  end

  def self.test_mode?
    fake_user_json.present?
  end

  def self.generate_fake_token
    OAuth2::AccessToken.new(client, SecureRandom.urlsafe_base64, expires_in: 2.hours)
  end

  class FailedToOpenConnection < Exception; end
end

require "go_sso/engine"
